@@ -26,38 +26,18 @@
 #include "mdadm.h"
 #include "md_p.h"
 
-/* from readme.c */
-mapping_t pers[] = {
-	{ "linear", -1},
-	{ "raid0", 0},
-	{ "0", 0},
-	{ "stripe", 0},
-	{ "raid1", 1},
-	{ "1", 1},
-	{ "mirror", 1},
-	{ "raid4", 4},
-	{ "4", 4},
-	{ "raid5", 5},
-	{ "5", 5},
-	{ "multipath", -4},
-	{ "mp", -4},
-	{ "raid6", 6},
-	{ "6", 6},
-	{ "raid10", 10},
-	{ "10", 10},
-	{ NULL, 0}
-};
+char const Name[] = "mdassemble";
 
 #ifndef MDASSEMBLE_AUTO
 /* from mdopen.c */
 int open_mddev(char *dev, int report_errors/*unused*/)
 {
-	int mdfd = open(dev, O_RDWR);
+	int mdfd = open(dev, O_RDONLY);
 	if (mdfd < 0)
-		fprintf(stderr, Name ": error opening %s: %s\n",
+		pr_err("error opening %s: %s\n",
 			dev, strerror(errno));
 	else if (md_get_version(mdfd) <= 0) {
-		fprintf(stderr, Name ": %s does not appear to be an md device\n",
+		pr_err("%s does not appear to be an md device\n",
 			dev);
 		close(mdfd);
 		mdfd = -1;
@@ -70,27 +50,16 @@ int create_mddev(char *dev, char *name, int autof/*unused*/, int trustworthy,
 	return open_mddev(dev, 0);
 }
 #endif
-int map_update(struct map_ent **mpp, int devnum, char *metadata,
-	       int *uuid, char *path)
-{
-	return 0;
-}
-struct map_ent *map_by_name(struct map_ent **mpp, char *name)
-{
-	return NULL;
-}
 
 int rv;
 int mdfd = -1;
-int runstop = 0;
-int readonly = 0;
-int verbose = 0;
-int force = 0;
 
-int main(int argc, char *argv[]) {
-	mddev_ident_t array_list =  conf_get_ident(NULL);
+int main(int argc, char *argv[])
+{
+	struct mddev_ident *array_list =  conf_get_ident(NULL);
+	struct context c = { .freeze_reshape = 1 };
 	if (!array_list) {
-		fprintf(stderr, Name ": No arrays found in config file\n");
+		pr_err("No arrays found in config file\n");
 		rv = 1;
 	} else
 		for (; array_list; array_list = array_list->next) {
@@ -105,9 +74,7 @@ int main(int argc, char *argv[]) {
 			if (mdfd >= 0)
 				close(mdfd);
 			rv |= Assemble(array_list->st, array_list->devname,
-				       array_list, NULL, NULL,
-				       readonly, runstop, NULL, NULL, 0,
-				       verbose, force);
+				       array_list, NULL, &c);
 		}
 	return rv;
 }
