@@ -144,7 +144,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 	struct createinfo *ci = conf_get_create_info();
 	int parts;
 	char *cname;
-	char devname[20];
+	char devname[37];
 	char devnm[32];
 	char cbuf[400];
 	if (chosen == NULL)
@@ -173,8 +173,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 			/* name *must* be mdXX or md_dXX in this context */
 			if (num < 0 ||
 			    (strcmp(cname, "md") != 0 && strcmp(cname, "md_d") != 0)) {
-				pr_err("%s is an invalid name "
-					"for an md device.  Try /dev/md/%s\n",
+				pr_err("%s is an invalid name for an md device.  Try /dev/md/%s\n",
 					dev, dev+5);
 				return -1;
 			}
@@ -191,13 +190,11 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 		 * empty.
 		 */
 		if (strchr(cname, '/') != NULL) {
-			pr_err("%s is an invalid name "
-				"for an md device.\n", dev);
+			pr_err("%s is an invalid name for an md device.\n", dev);
 			return -1;
 		}
 		if (cname[0] == 0) {
-			pr_err("%s is an invalid name "
-				"for an md device (empty!).", dev);
+			pr_err("%s is an invalid name for an md device (empty!).", dev);
 			return -1;
 		}
 		if (num < 0) {
@@ -226,8 +223,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 	if (name && name[0] == 0)
 		name = NULL;
 	if (name && trustworthy == METADATA && use_mdp == 1) {
-		pr_err("%s is not allowed for a %s container. "
-			"Consider /dev/md%d.\n", dev, name, num);
+		pr_err("%s is not allowed for a %s container. Consider /dev/md%d.\n", dev, name, num);
 		return -1;
 	}
 	if (name && trustworthy == METADATA)
@@ -322,7 +318,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 	else if (num < 0) {
 		/* need to choose a free number. */
 		char *_devnm = find_free_devnm(use_mdp);
-		if (devnm == NULL) {
+		if (_devnm == NULL) {
 			pr_err("No avail md devices - aborting\n");
 			return -1;
 		}
@@ -352,7 +348,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
 		if (lstat(devname, &stb) == 0) {
 			/* Must be the correct device, else error */
 			if ((stb.st_mode&S_IFMT) != S_IFBLK ||
-			    stb.st_rdev != (dev_t)devnm2devid(devnm)) {
+			    stb.st_rdev != devnm2devid(devnm)) {
 				pr_err("%s exists but looks wrong, please fix\n",
 					devname);
 				return -1;
@@ -420,9 +416,7 @@ int create_mddev(char *dev, char *name, int autof, int trustworthy,
  */
 int open_mddev(char *dev, int report_errors)
 {
-	int mdfd = open(dev, O_RDWR);
-	if (mdfd < 0 && errno == EACCES)
-		mdfd = open(dev, O_RDONLY);
+	int mdfd = open(dev, O_RDONLY);
 	if (mdfd < 0) {
 		if (report_errors)
 			pr_err("error opening %s: %s\n",
@@ -432,8 +426,7 @@ int open_mddev(char *dev, int report_errors)
 	if (md_get_version(mdfd) <= 0) {
 		close(mdfd);
 		if (report_errors)
-			pr_err("%s does not appear to be "
-				"an md device\n", dev);
+			pr_err("%s does not appear to be an md device\n", dev);
 		return -2;
 	}
 	return mdfd;
@@ -444,7 +437,7 @@ char *find_free_devnm(int use_partitions)
 	static char devnm[32];
 	int devnum;
 	for (devnum = 127; devnum != 128;
-	     devnum = devnum ? devnum-1 : (1<<20)-1) {
+	     devnum = devnum ? devnum-1 : (1<<9)-1) {
 
 		if (use_partitions)
 			sprintf(devnm, "md_d%d", devnum);
@@ -457,7 +450,7 @@ char *find_free_devnm(int use_partitions)
 		if (!use_udev()) {
 			/* make sure it is new to /dev too, at least as a
 			 * non-standard */
-			int devid = devnm2devid(devnm);
+			dev_t devid = devnm2devid(devnm);
 			if (devid) {
 				char *dn = map_dev(major(devid),
 						   minor(devid), 0);
